@@ -431,6 +431,7 @@ static int curl_download_internal(struct dload_payload *payload,
 	/* Ignore any SIGPIPE signals. With libcurl, these shouldn't be happening,
 	 * but better safe than sorry. Store the old signal handler first. */
 	mask_signal(SIGPIPE, SIG_IGN, &orig_sig_pipe);
+	dload_interrupted = 0;
 	mask_signal(SIGINT, &inthandler, &orig_sig_int);
 
 	/* perform transfer */
@@ -473,6 +474,13 @@ static int curl_download_internal(struct dload_payload *payload,
 						payload->remote_name, hostname);
 			}
 			goto cleanup;
+		case CURLE_COULDNT_RESOLVE_HOST:
+			payload->unlink_on_fail = 1;
+			handle->pm_errno = ALPM_ERR_SERVER_BAD_URL;
+			_alpm_log(handle, ALPM_LOG_ERROR,
+					_("failed retrieving file '%s' from %s : %s\n"),
+					payload->remote_name, hostname, error_buffer);
+			goto cleanup;
 		default:
 			/* delete zero length downloads */
 			if(fstat(fileno(localf), &st) == 0 && st.st_size == 0) {
@@ -526,7 +534,8 @@ static int curl_download_internal(struct dload_payload *payload,
 		if(payload->content_disp_name) {
 			/* content-disposition header has a better name for our file */
 			free(payload->destfile_name);
-			payload->destfile_name = get_fullpath(localpath, payload->content_disp_name, "");
+			payload->destfile_name = get_fullpath(localpath,
+				get_filename(payload->content_disp_name), "");
 		} else {
 			const char *effective_filename = strrchr(effective_url, '/');
 			if(effective_filename && strlen(effective_filename) > 2) {
@@ -578,7 +587,7 @@ cleanup:
 	unmask_signal(SIGINT, &orig_sig_int);
 	unmask_signal(SIGPIPE, &orig_sig_pipe);
 	/* if we were interrupted, trip the old handler */
-	if(dload_interrupted) {
+	if(dload_interrupted == ABORT_SIGINT) {
 		raise(SIGINT);
 	}
 
